package com.contextspace;

import static org.junit.Assert.*;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Test;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import com.contextspace.objectdb_example.dao.IAttributeNameDAO;
import com.contextspace.objectdb_example.dao.IHolderDAO;
import com.contextspace.objectdb_example.model.AttributeName;
import com.contextspace.objectdb_example.model.holder.Holder;

public class ExampleTest {

    private static ClassPathXmlApplicationContext context = null;
    
    // attribute
    private static IAttributeNameDAO attrNameDao;
    private static IHolderDAO holderDao;
    
    @BeforeClass
    public static void setUpBeforeClass() throws Exception {
        context = new ClassPathXmlApplicationContext(new String[] { "applicationContext-test.xml" });
        
        attrNameDao = context.getBean(IAttributeNameDAO.class);
        holderDao = context.getBean(IHolderDAO.class);
    }
    
    @AfterClass
    public static void tearDownAfterClass() throws Exception {
        if(context != null) {
            context.close();
        }      
        
        File f1 = new File("objectdb"+File.separator+"db");
        deleteDir(f1);
    }
    
    
    @Test
    public void test() throws Exception {
        
        assertEquals(0, attrNameDao.count());
        
        AttributeName an1 = new AttributeName("an1", "an1 eng");
        AttributeName an2 = new AttributeName("an2", "an2 eng");
        an2.setDisplayValue("fr", "an2 fr");
        AttributeName an3 = new AttributeName("an3", "an3 eng");
        an3.setDisplayValue("fr", "an3 fr");
        
        an1 = attrNameDao.create(an1);
        an2 = attrNameDao.create(an2);
        an3 = attrNameDao.create(an3);
        
        assertEquals(3, attrNameDao.count());
        
        assertEquals("<attributename name=\"an1\" values=\"en=an1 eng\" id=\"1\"/>", attrNameDao.find(an1.getId()).toString());
        assertEquals("<attributename name=\"an2\" values=\"fr=an2 fr,en=an2 eng\" id=\"2\"/>", attrNameDao.find(an2.getId()).toString());
        assertEquals("<attributename name=\"an3\" values=\"fr=an3 fr,en=an3 eng\" id=\"3\"/>", attrNameDao.find(an3.getId()).toString());
        
        
        Holder holder = new Holder();
        holder = holderDao.create(holder);
        assertEquals("<holder id=\"4\">null</holder>", holderDao.find(holder.getId()).toString());
        
        
        Holder holder2 = new Holder();
        holder2.addName(an1);
        
        holder2 = holderDao.create(holder2);
        
        //get it back using a full select... THIS WORKS
        assertEquals("<holder id=\"5\">null<list><attributename name=\"an1\" values=\"en=an1 eng\" id=\"1\"/></list></holder>", holderDao.lookupById(holder2.getId()).toString());
        
        //get it back using a find... THIS FAILS
        assertEquals("<holder id=\"5\">null<list><attributename name=\"an1\" values=\"en=an1 eng\" id=\"1\"/></list></holder>", holderDao.find(holder2.getId()).toString());

    }

    
    private static boolean deleteDir(File dir) {
        if (dir.isDirectory()) {
            String[] children = dir.list();
            for (int i=0; i<children.length; i++) {
                boolean success = deleteDir(new File(dir, children[i]));
                if (!success) {
                    return false;
                }
            }
        }
    
        // The directory is now empty so delete it
        return dir.delete();
    } 
}
